function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
 * Choice JS
*/
class Builder {
  //Constructor
  constructor(target, userOptions) {
    this.target = target; //Build our default options

    this.options = _objectSpread(_objectSpread({}, this.buildDefaultOptions(userOptions)), userOptions);
  }
  /**
   * Create a new choice dropdown
   * 
   * @return {Promise}
  */


  create() {
    var _this = this;

    return _asyncToGenerator(function* () {
      //Build it
      var choice = yield _this.build(); //Toast bindings

      yield _this.bindings(choice); //Configure the select based on the options

      yield _this.configureOptions(choice); //Hide the current

      _this.target.classList.add('choicejs-hidden-select'); //Deploy the select


      _this.target.parentNode.insertBefore(choice, _this.target.nextSibling); //Done


      return choice;
    })();
  }
  /**
   * Build the choice
   * 
   * @return {Promise} HTMLElement The new choice
  */


  build() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      //Build the new Select HTML element and add it directly after the target
      var wrapper = document.createElement('div'),
          select = document.createElement('div'),
          search = document.createElement('input'),
          dropdown = document.createElement('div'),
          list = document.createElement('ul'); //Set attributes

      wrapper.classList.add('choicejs');
      select.classList.add('choicejs-select');
      select.innerHTML = _this2.options.placeholder;
      search.classList.add('choicejs-search');
      search.setAttribute('placeholder', 'Search');
      dropdown.classList.add('choicejs-dropdown'); //Loop the targets options and build our list

      for (var option of _this2.target.querySelectorAll('option')) {
        var item = document.createElement('li');
        item.setAttribute('value', option.value);
        item.innerHTML = option.innerHTML;
        list.appendChild(item);
      } //Build the whole select and return it


      dropdown.appendChild(search);
      dropdown.appendChild(list);
      wrapper.appendChild(select);
      wrapper.appendChild(dropdown); //Return

      return wrapper;
    })();
  }
  /**
   * Configure options for the choice
   * 
   * @param {HTMLElement} choice - The select we're configuring options for
   * @return {Promise}
  */


  configureOptions(choice) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      //Width and height
      choice.style.width = _this3.options.width == 'inherit' ? _this3.target.offsetWidth + 'px' : _this3.options.width, choice.style.minHeight = _this3.options.height == 'inherit' ? _this3.target.offsetHeight + 'px' : _this3.options.height; //Class and id

      if (_this3.options.id != '') {
        choice.setAttribute('id', _this3.options.id);
      }

      if (_this3.options.class != '') {
        choice.classList.add(_this3.options.class);
      } //Return


      return;
    })();
  }
  /**
   * Build the default options
   * 
   * @param {Object} userOptions The user options for any overwrites
   * @return {Object} An object of options
  */


  buildDefaultOptions(userOptions) {
    var options = {
      width: 'inherit',
      height: 'inherit',
      placeholder: this.target.hasAttribute('placeholder') ? this.target.getAttribute('placeholder') : 'Select...',
      id: '',
      class: '',
      closeOnSelect: false
    }; //Return

    return options;
  }
  /**
   * Add all of the bindings for the choice
   * 
   * @param {HTMLElement} choice - The select we're building the bindings for
   * @return {Promise}
  */


  bindings(choice) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      //Return
      return yield Promise.all([_this4.bindActiveListener(choice), _this4.bindOptionFiltering(choice), _this4.bindOptionSelect(choice)]);
    })();
  }
  /**
   * Bind active click
   *
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindActiveListener(choice) {
    return _asyncToGenerator(function* () {
      choice.addEventListener('click', () => {
        choice.classList.add('active');
      });
      document.addEventListener('click', function (event) {
        if (!choice.contains(event.target)) {
          choice.classList.remove('active');
        }
      }); //Return

      return;
    })();
  }
  /**
   * Bind option select
   * 
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindOptionSelect(choice) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      //Single or multiple select?
      var multiple = _this5.target.getAttribute('multiple'),
          bindings = []; //Loop options and bind


      for (var option of choice.querySelectorAll('ul li')) {
        if (multiple && multiple == 'true') {
          bindings.push(_this5.bindMultipleOptionSelect(option));
        } else {
          bindings.push(_this5.bindSingleOptionSelect(option));
        }
      } //Return


      return Promise.all(bindings);
    })();
  }
  /**
   * Single option click
   *
   * @param {HTMLElement} option - The option
   * @returns {Promise}
  */


  bindSingleOptionSelect(option) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      //Elems
      var choice = option.closest('.choicejs'),
          select = choice.querySelector('.choicejs-select'); //Add event listener

      option.addEventListener('click', /*#__PURE__*/function () {
        var _ref = _asyncToGenerator(function* (e) {
          e.stopPropagation();
          e.stopImmediatePropagation(); //Get the value we've selected

          var value = option.getAttribute('value'); //Take off all siblings

          for (var sibling of choice.querySelectorAll('li')) {
            sibling.classList.remove('selected');
          } //Toggle the selected class


          option.classList.toggle('selected'); //Set the value

          select.innerHTML = option.innerHTML;

          for (var hiddenOption of _this6.target.querySelectorAll('option')) {
            hiddenOption.removeAttribute('selected');
          }

          _this6.target.querySelector("option[value=\"".concat(value, "\"]")).setAttribute('selected', true); //Close on select?


          if (_this6.options.closeOnSelect) {
            choice.classList.remove('active');
          }
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      }()); //Return

      return;
    })();
  }
  /**
   * Multiple option click
   *
   * @param {HTMLElement} option - The option
   * @returns {Promise}
  */


  bindMultipleOptionSelect(option) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      //Elems
      var choice = option.closest('.choicejs'),
          select = choice.querySelector('.choicejs-select'); //Add event listener

      option.addEventListener('click', /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator(function* (e) {
          e.stopPropagation();
          e.stopImmediatePropagation(); //Get the value we've selected

          var value = option.getAttribute('value'); //First check if the option is already selected

          if (option.classList.contains('selected')) {
            //Remove
            _this7.target.querySelector("option[value=\"".concat(value, "\"]")).removeAttribute('selected');

            select.querySelector(".choicejs-pill[value=\"".concat(value, "\"]")).remove();
          } else {
            //Select it
            _this7.target.querySelector("option[value=\"".concat(value, "\"]")).setAttribute('selected', true);

            var pill = yield _this7.buildOptionPill(option); //Bind the remove button

            yield _this7.bindPillRemoveBtn(pill, choice); //Append it

            select.appendChild(pill);
          } //Toggle the selected class


          option.classList.toggle('selected'); //Close on select?

          if (_this7.options.closeOnSelect) {
            choice.classList.remove('active');
          }
        });

        return function (_x2) {
          return _ref2.apply(this, arguments);
        };
      }()); //Return

      return;
    })();
  }
  /**
   * Build option pill
   *
   * @param {Node} option - The option selected
   * @returns {Promise} A node element
  */


  buildOptionPill(option) {
    return _asyncToGenerator(function* () {
      var wrapper = document.createElement('div'),
          close = document.createElement('span'),
          value = option.getAttribute('value'); //Attributes

      wrapper.classList.add('choicejs-pill');
      wrapper.setAttribute('value', value);
      wrapper.innerHTML = option.innerHTML; //Build it

      wrapper.appendChild(close); //Return

      return wrapper;
    })();
  }
  /**
   * Bind option filtering
   * 
   * @param {HTMLElement} choice - The choice we're binding
   * @return {Promise}
  */


  bindOptionFiltering(choice) {
    return _asyncToGenerator(function* () {
      //When the choice search bar changes
      var searchTimeout = 0,
          searchBar = choice.querySelector('.choicejs-search'),
          options = choice.querySelectorAll('ul li'); //When the postcode is changed, test it

      searchBar.addEventListener('input', () => {
        //Clear the above timeout
        clearTimeout(searchTimeout); //Set a new timeout

        searchTimeout = setTimeout(() => {
          //Loop all options
          for (var option of options) {
            if (option.innerHTML.includes(searchBar.value)) {
              option.classList.remove('hide');
            } else {
              option.classList.add('hide');
            }
          }
        }, 500);
      }); //Return

      return;
    })();
  }
  /**
   * Bind pill remove btn
   *
   * @param {Node} pill - The pill
   * @param {HTMLElement} choice - The choice instance
   * @returns {Promise}
  */


  bindPillRemoveBtn(pill, choice) {
    var _this8 = this;

    return _asyncToGenerator(function* () {
      //Params
      var value = pill.getAttribute('value'); //Add the remove event listener

      pill.querySelector('span').addEventListener('click', e => {
        e.stopPropagation();
        e.stopImmediatePropagation();

        _this8.target.querySelector("option[value=\"".concat(value, "\"]")).removeAttribute('selected');

        choice.querySelector(".choicejs-dropdown li[value=\"".concat(value, "\"]")).classList.remove('selected');
        pill.remove();
      }); //Return

      return;
    })();
  }

}
/**
 * Export the choice function to use
 *
 * @param {NodeElement} target - The target we're adding the choice to
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of choice
 */


export var Choice = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (target, options) {
    //Start a new Choice
    var newBuild = new Builder(target, options); //Run the build

    return yield newBuild.create();
  });

  return function Choice(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();